#include "minunit.h"
#include "iconvert.h"
#include <stdio.h>

int tests_run = 0;

char* utf8_text = "这是一段UTF-8文本.";
char* gb2312_text = "һUTF-8ı.";

char* test_init() {
    iconv_info* info;
    mu_assert((info = iconv_create("gb12")) == 0); /* wrong encoding */

    mu_assert(info = iconv_create("gb2312"));
    mu_assert(strcmp(info->m_encoding, "gb2312") == 0);
    mu_assert(info->m_iconv_from_unicode != (iconv_t)(-1));
    mu_assert(info->m_iconv_to_unicode != (iconv_t)(-1));
    mu_assert(info->m_buffer);

    mu_assert(iconv_destory(info) == 0);
    return 0;
}

char* test_convert() {
    char buf[100];
    uchar_t u_buf[100];

    iconv_info* info = iconv_create("UTF-8");
    iconv_info* info2 = iconv_create("GB2312");

    mu_assert(info);
    mu_assert(info2);
    mu_assert(iconv_to_unicode(info, utf8_text, strlen(utf8_text), u_buf, 2) == -2);
    mu_assert(iconv_to_unicode(info, utf8_text, strlen(utf8_text), u_buf, 11) == -2);
    mu_assert(iconv_to_unicode(info, utf8_text, strlen(utf8_text), u_buf, 12) == -2);
    mu_assert(iconv_to_unicode(info, utf8_text, strlen(utf8_text), u_buf, 13) == 12);
    mu_assert(iconv_to_unicode(info, utf8_text, strlen(utf8_text), u_buf, 100) == 12);
    mu_assert(iconv_from_unicode(info2, u_buf, 12, buf, 2) == -2);
    mu_assert(iconv_from_unicode(info2, u_buf, 12, buf, 17) == -2);
    mu_assert(iconv_from_unicode(info2, u_buf, 12, buf, 18) == -2);
    mu_assert(iconv_from_unicode(info2, u_buf, 12, buf, 19) == 18);
    mu_assert(iconv_from_unicode(info2, u_buf, 12, buf, 100) == 18);
    mu_assert(strcmp(buf, gb2312_text) == 0);

    mu_assert(iconv_destory(info) == 0);

    /* test str_to_unicode */
    mu_assert(ustrlen(str_to_unicode("", "UTF-8")) == 0);
    mu_assert(ustrlen(str_to_unicode("A", "UTF-8")) == 1);
    mu_assert(str_to_unicode("A", "UTF-8")[0] == L'A');
    mu_assert(ustrlen(str_to_unicode(utf8_text, "UTF-8")) == 12);
    mu_assert(str_to_unicode(utf8_text, "UTF-8")[4] == L'U');

    return 0;
}

static char * all_tests() {
     mu_run_test(test_init);
     mu_run_test(test_convert);
     return 0;
 }
 
int main(int argc, char **argv) {
    char *result = all_tests();
    if (result != 0) {
        printf("%s\n", result);
    }
    else {
        printf("ALL TESTS PASSED\n");
    }
    printf("Tests run: %d\n", tests_run);

    return result != 0;
}

