#include <stdio.h>
#include "minunit.h"
#include "unicode.h"

int tests_run = 0;

char* test_ustrlen() {
    uchar_t* s1;
    uchar_t s2[] = {0x0001, 0x0002, 0x0000};
    s1 = L"";
    mu_assert(ustrlen(s1) == 0);
    mu_assert(ustrlen(s2) == 2);
    return 0;
}

char* test_ustrdup() {
    uchar_t* s1 = L"";
    uchar_t s2[] = {0x0001, 0x0002, 0x0000};
    uchar_t* s3;
    s3 = ustrdup(s1);
    mu_assert(*s3 == L'\0');
    s3 = ustrdup(s2);
    mu_assert(ustrlen(s3) == ustrlen(s2));
    mu_assert(s3[0] == s2[0]);
    mu_assert(s3[1] == s2[1]);
    mu_assert(s3[2] == s2[2]);
    return 0;
}

char* test_ustrcpy() {
    uchar_t* s1 = L"";
    uchar_t s2[] = {0x0001, 0x0002, 0x0000};
    uchar_t s3[] = {0x0002, 0x0001, 0x0000};
    uchar_t* s4;

    s4 = ustrcpy(s3, s1);
    mu_assert(*s3 == L'\0');
    mu_assert(*s4 == L'\0');

    s4 = ustrcpy(s3, s2);
    mu_assert(ustrlen(s3) == ustrlen(s2));
    mu_assert(ustrlen(s4) == ustrlen(s2));
    mu_assert(s3[0] == s2[0]);
    mu_assert(s3[1] == s2[1]);
    mu_assert(s3[2] == s2[2]);
    return 0;
}

char* test_ustrncpy() {
    uchar_t* s1 = L"";
    uchar_t s2[] = {0x0001, 0x0002, 0x0000};
    uchar_t s3[] = {0x0002, 0x0001, 0x0001, 0x0003};
    uchar_t* s4;

    s4 = ustrncpy(s3, s2, 0);
    mu_assert(s4[0] == 0x0002);

    s4 = ustrncpy(s3, s2, 1);
    mu_assert(s4[0] == 0x0001);
    mu_assert(s4[1] == 0x0001);

    s4 = ustrncpy(s3, s2, 2);
    mu_assert(s4[0] == 0x0001);
    mu_assert(s4[1] == 0x0002);

    s4 = ustrncpy(s3, s2, 3);
    mu_assert(s4[0] == 0x0001);
    mu_assert(s4[1] == 0x0002);
    mu_assert(s4[2] == 0x0000);

    s4 = ustrncpy(s3, s2, 4);
    mu_assert(s4[0] == 0x0001);
    mu_assert(s4[1] == 0x0002);
    mu_assert(s4[2] == 0x0000);
    mu_assert(s4[3] == 0x0000);

    return 0;
}



static char * all_tests() {
     mu_run_test(test_ustrlen);
     mu_run_test(test_ustrdup);
     mu_run_test(test_ustrcpy);
     mu_run_test(test_ustrncpy);
     return 0;
 }
 
int main(int argc, char **argv) {
    char *result = all_tests();
    if (result != 0) {
        printf("%s\n", result);
    }
    else {
        printf("ALL TESTS PASSED\n");
    }
    printf("Tests run: %d\n", tests_run);

    return result != 0;
}

