/*
 * vi:ts=4:tw=78:shiftwidth=4:expandtab
 * vim600:fdm=marker
 *
 * unicode.c  -  description
 *
 * Copyright (C) 2004 by Zhang Le <ejoy@users.sourceforge.net>
 * Begin       : 05-Aug-2004
 * Last Change : 17-Aug-2004.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <assert.h>
#include "unicode.h"

size_t ustrlen(const uchar_t* str) {
    size_t len = 0;
    const uchar_t* s = str;
    assert(s);
    while (*s++) {
        ++len;
    }
    return len;
}

uchar_t* ustrdup(const uchar_t* s) {
    size_t len;
    uchar_t* p;
    assert(s);
    len = ustrlen(s);
    p = (uchar_t*)malloc(sizeof(uchar_t)*(len+1));
    assert(p);
    ustrcpy(p, s);
    return p;
}

uchar_t* ustrcpy(uchar_t* dest, const uchar_t* src) {
    const uchar_t* s = src;
    uchar_t* d = dest;
    assert(s);
    while (*s) {
        *d++ = *s++;
    }
    *d = L'\0';
    return dest;
}

uchar_t* ustrncpy(uchar_t* dest, const uchar_t *src, size_t count) {
    const uchar_t* s = src;
    uchar_t* d = dest;
    assert(s);
    while (*s && count > 0) {
        *d++ = *s++;
        --count;
    }
    while (count > 0) {
        *d++ = L'\0';
        --count;
    }
    return dest;
}

/*  int ustrcmp(const uchar_t* __s1, const uchar_t* __s2, size_t __n) {*/
/*    for (size_t __i = 0; __i < __n; ++__i)*/
/*      if (!eq(__s1[__i], __s2[__i]))*/
/*        return __s1[__i] < __s2[__i] ? -1 : 1;*/
/*    return 0;*/
/*  }*/

